﻿# Iterates over all web apps (or a single web app),
# finds all occurrences of the specified feature id and upgrades them to the newest/latest version.
# Must be launched with -Debug:$true

param 
(
    [parameter(Mandatory=$true)][string]$param_FeatureIdString,
	[parameter(Mandatory=$false)][string]$param_WebAppUrl
)

if (-not (Get-PSSnapin -Name Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue)) {
    Add-PSSnapin Microsoft.SharePoint.PowerShell
}

$VERBOSE = $null # prints logfile output to console
$DEBUG   = $null # does not execute any update or delete actions

# Can overwrite default values with switch like:   -Debug:$false
if (!$PSBoundParameters.TryGetValue('Verbose', [ref]$VERBOSE)) { $VERBOSE = $true }
if (!$PSBoundParameters.TryGetValue('Debug',   [ref]$DEBUG))   { $DEBUG   = $true }

# Constant values
$global:execFilePath = $MyInvocation.MyCommand.Definition

# Parsed values
[System.Guid]$param_FeatureId = $param_FeatureIdString


##############################################################


# Gets the full path of the logfile for the current script (which is always [ScriptPath]\[ScriptName].log)
Function GetLogFilePath
{
    return $global:execFilePath.SubString(0, $global:execFilePath.LastIndexOf(".")) + ".log"
}


# Gets the name of the script
Function GetScriptName
{
    return $global:execFilePath.Remove(0, $global:execFilePath.LastIndexOf("\") + 1)
}


# Appends a log message to the file specified in global:logPath
Function WriteLog ($logType, $webUrl, $logMessage_1, $logMessage_2)
{
    $msg = [string]::Format(
        "{0}`t{1}`t{2}`t""{3}""`t""{4}""",
        (Get-Date).ToString("yyyy-MM-ddTHH:mm:ss:fff"),
        $logType,
        $webUrl,
        ($logMessage_1 -replace '"', '""'),
        ($logMessage_2 -replace '"', '""'))

    Write-Output $msg >> $global:logPath
}


# Writes a message to console output (if verbose mode) and logs the original message
Function WriteMessage($logType, $webUrl, $msg1, $msg2, [int]$consoleTabLevel = 0)
{
    $tabsPrefix = ""

    for($i=0; $i -lt $consoleTabLevel; $i++)
    {
        $tabsPrefix = $tabsPrefix + "`t"
    }

    if ($VERBOSE)
    {
        Write-Host "$($tabsPrefix)$($msg2)"
    }

    WriteLog $logType $webUrl $msg1 $msg2
}


# Checks whether the specified string is neither null nor empty
Function StringNotNullOrEmpty([string]$str)
{
    return ($str -ne $null -and $str.Trim().Length -gt 0)
}


# Iterates web application by url
Function IterateWebApplication ([string]$webAppUrl)
{
    $webApp = Get-SPWebApplication $webAppUrl

    if ($webApp -eq $null)
    {
        return
    }

    IterateWebApp $webApp
}


# Iterates web application
Function IterateWebApp($webApp)
{
    WriteMessage "INFO" $webApp.Url "IterateWebApp" "Iterating through web application '$($webApp.Url)'..."

	try
    {
        foreach ($feature in $webApp.QueryFeatures($param_FeatureId, $true))
        {
            WriteMessage "INFO" "$($feature.Parent.Url)" "IterateWebApp" "Upgrading '$($feature.Definition.DisplayName)' in '$($feature.Parent.Url)' from $($feature.Version.ToString()) to $($feature.Definition.Version.ToString())." 1

            if (-not $DEBUG)
            {
                $feature.Upgrade($false)
            }
        }
    }
    catch
    {
        Write-Error $_.Exception
        pause
        throw
    }
}


# Main
$global:logPath = (GetLogFilePath)

# Reset log file
if (Test-Path $global:logPath)
{
    Remove-Item $global:logPath
}

# Print start message
if ($DEBUG)
{
    WriteMessage "INFO" "" "" "-- Start script execution DEBUG/TEST Mode: $(GetScriptName) --"
}
else
{
    WriteMessage "INFO" "" "" "-- Start script execution: $(GetScriptName) --"
}

# Iterate
if (StringNotNullOrEmpty $param_WebAppUrl)
{
    IterateWebApplication $param_WebAppUrl
}
else
{
    foreach ($webApp in Get-SPWebApplication)
    {
        IterateWebApp $webApp
    }
}
WriteMessage "INFO" "" "" "-- End script execution --"